@@ -1,11 +1,9 @@
 use anyhow::Result;
 use console::style;
 use crossbeam_channel::{select, Receiver, Sender};
-use indicatif::{MultiProgress, ProgressBar, ProgressDrawTarget, ProgressStyle};
-use lazy_static::lazy_static;
+use indicatif::{MultiProgress, ProgressBar, ProgressDrawTarget};
 use std::{
     collections::HashSet,
-    fs,
     path::{Path, PathBuf},
     thread::{self, JoinHandle},
     time::Instant,
@@ -16,28 +14,19 @@ use crate::{
     files::GlobFilesReader,
     hashes, lock,
     manifest::{store_and_clean_results, Manifest},
-    outputs::restore_outputs,
+    outputs::{clean_latest_outputs, restore_outputs},
     tasks::{Task, TaskContext, TaskGraph},
-    util, Config, FasterError,
+    util::{self, DONE_STYLE, PROGRESS_STYLE, STATUS_STYLE},
+    Config, FasterError,
 };
 
-lazy_static! {
-    static ref PROGRESS_STYLE: ProgressStyle =
-        ProgressStyle::with_template("  {spinner} {wide_msg}")
-            .expect("failed to create progress style")
-            .tick_chars("⠋⠙⠹⠸⠼⠴⠦⠧⠇⠏");
-    static ref STATUS_STYLE: ProgressStyle = ProgressStyle::with_template("  {prefix} {wide_msg}")
-        .expect("failed to create status style");
-    static ref DONE_STYLE: ProgressStyle =
-        ProgressStyle::with_template("  {prefix} {wide_msg}").expect("failed to create done style");
-}
-
 const FASTER_ROOT_VAR: &str = "FASTER_ROOT";
 const FASTER_TASK_VAR: &str = "FASTER_TASK";
 
 struct TaskMessage(String, Task);
 
-enum CommandResult {
+#[derive(Debug)]
+pub enum CommandResult {
     Success(String),
     Failed(String),
     CachedFailed(String),
@@ -54,6 +43,7 @@ pub fn run<P>(
     draw_target: ProgressDrawTarget,
     job_count: usize,
     ctrlc_rx: Receiver<()>,
+    disable_cache_global: bool,
 ) -> Result<()>
 where
     P: AsRef<Path>,
@@ -71,6 +61,8 @@ where
     let (done_tx, done_rx) = crossbeam_channel::unbounded::<(String, Result<CommandResult>)>();
     let (abort_tx, abort_rx) = crossbeam_channel::bounded::<()>(job_count);
 
+    let disable_cache_global = disable_cache_global || config.disable_cache;
+
     let worker_handles = (0..job_count)
         .map(|_| {
             let job_rx = job_rx.clone();
@@ -88,6 +80,7 @@ where
                     job_rx.clone(),
                     done_tx.clone(),
                     abort_rx.clone(),
+                    disable_cache_global,
                 )
             })
         })
@@ -195,6 +188,7 @@ fn run_worker<P>(
     rx: Receiver<TaskMessage>,
     done_tx: Sender<(String, Result<CommandResult>)>,
     abort_rx: Receiver<()>,
+    disable_cache_global: bool,
 ) where
     P: AsRef<Path>,
 {
@@ -202,9 +196,18 @@ fn run_worker<P>(
 
     while let Ok(TaskMessage(task_name, task)) = rx.recv() {
         let bar = progress.add(ProgressBar::new(1));
+        let disable_cache = disable_cache_global || task.disable_cache;
 
         let started = Instant::now();
-        let result = run_task(&workdir, &bar, &task_name, &task, global_env, &abort_rx);
+        let result = run_task(
+            &workdir,
+            &bar,
+            &task_name,
+            &task,
+            global_env,
+            disable_cache,
+            &abort_rx,
+        );
 
         match &result {
             Ok(CommandResult::Cancelled) => {
@@ -286,12 +289,13 @@ fn run_worker<P>(
     }
 }
 
-fn run_task<P>(
+pub fn run_task<P>(
     workdir: P,
     bar: &ProgressBar,
     task_name: &str,
     task: &Task,
     global_env: &ResolvedEnv,
+    disable_cache: bool,
     abort_rx: &Receiver<()>,
 ) -> Result<CommandResult>
 where
@@ -327,27 +331,21 @@ where
     // We're about to either restore old outputs or new create new inputs
     // by running the defined script. This means that we should first look
     // if we had previous outputs and remove them.
-    if let Some(previous_outputs) = manifest
-        .get_latest_run()
-        .and_then(|run| run.outputs.as_ref())
-    {
-        // These were previously added to the archive, so they are guaranteed to
-        // be files. Just try to delete them 1 by 1 and ignore failures.
-        for path in previous_outputs {
-            let _ = fs::remove_file(path);
-        }
-    }
+    clean_latest_outputs(&ctx, &manifest);
 
     let current_hash = hashes::calc_hash::<GlobFilesReader>(global_env, &ctx)?;
-    if let Some(stored_run) = manifest.runs.iter().find(|run| run.sum == current_hash) {
-        if stored_run.success {
-            restore_outputs(&ctx, stored_run.sum)?;
-            return Ok(CommandResult::Cached);
-        }
 
-        // The last run was found, and failed. Retrieve the logs and
-        // return them again.
-        return Ok(CommandResult::CachedFailed(String::new()));
+    if !disable_cache {
+        if let Some(stored_run) = manifest.runs.iter().find(|run| run.sum == current_hash) {
+            if stored_run.success {
+                restore_outputs(&ctx, stored_run.sum)?;
+                return Ok(CommandResult::Cached);
+            }
+
+            // The last run was found, and failed. Retrieve the logs and
+            // return them again.
+            return Ok(CommandResult::CachedFailed(String::new()));
+        }
     }
 
     if task.script.is_none() {
@@ -366,6 +364,7 @@ where
         .unchecked();
 
     command = command
+        .dir(workdir.as_ref())
         .env(FASTER_ROOT_VAR, workdir.as_ref().as_os_str())
         .env(FASTER_TASK_VAR, &ctx.task_name);
 
@@ -430,8 +429,61 @@ where
 
 #[cfg(test)]
 mod tests {
+    use crossbeam_channel::bounded;
+
     use super::*;
-    use crate::util::tests::run_in_temp_dir;
+    use crate::util::tests::{run_in_temp_dir, touch_file};
+
+    #[test]
+    fn clean() {
+        let workdir = run_in_temp_dir();
+        let (_, abort_rx) = bounded::<()>(1);
+        let bar = ProgressBar::hidden();
+
+        let env = ResolvedEnv::default();
+        let task_name = "test";
+        let task = Task {
+            script: Some(
+                r#"
+                # check if the files exist and error out if they do
+                [[ -f foo ]] && exit 1
+                [[ -d bar ]] && exit 1
+                [[ -f bar/baz ]] && exit 1
+
+                touch foo
+                mkdir bar
+                touch bar/baz
+            "#
+                .to_owned(),
+            ),
+            environment: env.clone(),
+            outputs: vec!["foo".to_owned(), "bar".to_owned()],
+            ..Task::default()
+        };
+
+        // Task should fail if the files exist
+        touch_file(workdir.as_ref().join("foo"));
+        touch_file(workdir.as_ref().join("bar/baz"));
+
+        let result = run_task(&workdir, &bar, task_name, &task, &env, true, &abort_rx);
+        assert!(matches!(result, Ok(CommandResult::Failed(..))));
+
+        // Use a new dir here because we can't rely on the files getting deleted immediately
+        let workdir = run_in_temp_dir();
+
+        // Task should succeed in empty dir
+        let result = run_task(&workdir, &bar, task_name, &task, &env, true, &abort_rx);
+        assert!(matches!(result, Ok(CommandResult::Success(..))));
+        assert!(workdir.as_ref().join("foo").is_file());
+        assert!(workdir.as_ref().join("bar/baz").is_file());
+
+        // Another run should succeed as well, clearing the directory before running the script
+        // and then outputting the files again
+        let result = run_task(&workdir, &bar, task_name, &task, &env, true, &abort_rx);
+        assert!(matches!(result, Ok(CommandResult::Success(..))));
+        assert!(workdir.as_ref().join("foo").is_file());
+        assert!(workdir.as_ref().join("bar/baz").is_file());
+    }
 
     #[test]
     fn run_single() {
@@ -457,6 +509,7 @@ mod tests {
             indicatif::ProgressDrawTarget::hidden(),
             1,
             rx,
+            false,
         );
 
         assert!(matches!(result, Ok(())));
@@ -485,6 +538,7 @@ mod tests {
             indicatif::ProgressDrawTarget::hidden(),
             1,
             rx,
+            false,
         );
 
         assert!(matches!(result, Ok(())));
@@ -523,6 +577,7 @@ mod tests {
             indicatif::ProgressDrawTarget::hidden(),
             1,
             rx,
+            false,
         );
 
         assert!(matches!(result, Ok(())));
@@ -546,6 +601,7 @@ mod tests {
             indicatif::ProgressDrawTarget::hidden(),
             1,
             rx,
+            false,
         );
 
         assert!(matches!(result, Ok(())));
